@@ -20,7 +20,7 @@
 
 import traceback
 
-from typing import Callable, List, Optional
+from typing import Callable, Iterator, List, Optional
 
 from .exceptions import (IptablesError, IptablesParsingError)
 from .rule import ChainRule
@@ -35,6 +35,8 @@ class Chain:
     """This class is used to represent an iptables chain.
     A chain contains a list of rules which can be referenced by
     number (rule numbers start with 1).
+
+    A :class:`Chain` object is iterable, returning the chain's rules.
     """
 
     def __init__(self, chain_name: str):
@@ -51,11 +53,21 @@ class Chain:
     def __str__(self):
         return f'Chain({self.__real_chain_name})'
 
+    def __iter__(self):
+        """Iterator for the chain's rules
+        """
+        return iter(self.__rule_list)
+
     def is_builtin(self) -> bool:       # pylint: disable=no-self-use
         """Returns ``True`` if this is a built-in chain (e.g. ``INPUT``)
         """
         return False
 
+    def has_rules(self) -> bool:
+        """Returns ``True`` if the chain contains any rules
+        """
+        return bool(self.__rule_list)
+
     def get_reference_count(self) -> int:
         """Returns the reference count of a (non-builtin) chain; returns 0
         for builtin chains
@@ -101,11 +113,67 @@ class Chain:
                 count += 1
         return count
 
+    def get_rule_count(self) -> int:
+        """Returns the number of rules in the chain
+        """
+        return len(self.__rule_list)
+
     def get_rules(self) -> List[ChainRule]:
         """Returns a list that contains the chain rules.
         """
         return self.__rule_list[:]
 
+    def iter_rules(self, *, chain_target=False, uses_goto=False,
+                    match_count: Optional[int] =None,
+                    match: Optional['Match'] =None) -> Iterator[ChainRule]:
+        """Returns an iterator for the chain rules. The rules returned
+        by the iterator depend on the arguments:
+
+        :param chain_target: if ``True``, return rules where the target
+            is a chain
+        :param uses_goto: if ``True``, return rules that use goto
+        :param match_count: if not ``None``, return rules that have that
+            number of matches
+        :param match: if not ``None``, return rules that have a matching
+            :class:`Match` in their match list
+        """
+        if (chain_target or uses_goto or match_count is not None or
+                                                match is not None):
+            match_klass = type(None)
+            if match:
+                # Perform a match class comparison
+                match_klass = type(match)
+                for crit in match.get_criteria():
+                    if crit is not None and crit.is_set():
+                        # Perform a match value comparison
+                        match_klass = type(None)
+                        break
+            # Define the filter function
+            def rule_filter(    # pylint: disable=too-many-return-statements
+                        rule: ChainRule) -> bool:
+                """Returns True/False based on whether the specified rule
+                satisfies the specified criteria.
+                """
+                if chain_target or uses_goto:
+                    target_chain = rule.get_target_chain()
+                    if target_chain is None:
+                        return False
+                    if uses_goto and not rule.uses_goto():
+                        return False
+                if (match_count is not None and
+                                        rule.get_match_count() != match_count):
+                    return False
+                if match is None:
+                    return True
+                for rule_match in rule:
+                    if match_klass is type(rule_match):
+                        return True
+                    if rule_match == match:
+                        return True
+                return False
+            return filter(rule_filter, self.__rule_list)
+        return iter(self.__rule_list)
+
     def _set_rule_list(self, rule_list: List[ChainRule]) -> None:
         """Set the rule list.
         This method is only used by the parsing code, so it does not
@@ -530,13 +598,12 @@ class Chain:
         """Parse a set of lines from the output of ``iptables -xnv``
         into a :class:`Chain` object.
 
-        It returns a :class:`Chain` object.
-
         It raises an :exc:`IptablesParsingError` if there is a parsing error.
 
         :param line_list: list of **iptables(8)** output lines
         :param pft: an :class:`IptablesPacketFilterTable` object
         :param log_parsing_failures: if ``True``, log any parsing failures
+        :rtype: a :class:`Chain` object
         """
         line_iter = iter(line_list)
         chain = cls.__parse_chain_line(next(line_iter).rstrip())
@@ -592,7 +659,8 @@ class BuiltinChain(Chain):
         return f'BuiltinChain({self.__real_chain_name})'
 
     def is_builtin(self) -> bool:
-        """Always ``True``
+        """
+        :rtype: always returns ``True``
         """
         return True
 
