@@ -22,7 +22,7 @@ import logging
 import subprocess
 import time
 
-from typing import Dict, Iterator, List, Optional, Tuple
+from typing import Iterator, List, Mapping, Optional, Tuple
 
 from .exceptions import (
                 IptablesError, IptablesParsingError, IptablesExecutionError)
@@ -78,13 +78,17 @@ class IptablesPacketFilterTable:
 
     __CHAIN_NAME_LIMIT = 28
 
-    def __init__(self, table_name: str, chain_prefix: str =None, runner=None):
+    def __init__(self, table_name: str, chain_prefix: Optional[str] =None,
+                        runner=None):
         """
         :param table_name: one of ``filter``, ``mangle``, ``nat``, ``raw``,
             or ``security``
-        :param chain_prefix: if specified, only chains with names starting
-            with this prefix will be included (however, the builtin chains
-            are **always** included)
+        :param chain_prefix: if specified, user chains are split into two
+            groups: those with names starting with the prefix and those
+            that don't) with methods like :meth:`get_user_chains`
+            returning only these user chains;
+            the chain name that excludes the prefix is referred to as
+            the **logical** chain name
         :param runner: optional Callable object used to invoke the
             **iptables(8)** command. The object should have the same signature
             as :func:`subprocess.run` and provide the same functionality.
@@ -145,7 +149,7 @@ class IptablesPacketFilterTable:
         return self.__prefix
 
     def set_prefix(self, prefix: Optional[str]) -> None:
-        """Partition (or repartition) the chains according to the
+        """Partition (or repartition) the user chains according to the
         specified prefix.
 
         :param prefix: the new prefix
@@ -241,7 +245,7 @@ class IptablesPacketFilterTable:
         """
         return self.__rcn(lcn)
 
-    def get_chain_map(self) -> Dict[str, Chain]:
+    def get_chain_map(self) -> Mapping[str, Chain]:
         """Returns a dictionary containing all accessible chains;
         the key is the real chain name (if the
         :class:`IptablesPacketFilterTable` uses a prefix, only chains
@@ -478,7 +482,7 @@ class IptablesPacketFilterTable:
             builtin_chain._set_stats(    # pylint: disable=protected-access
                         chain_packet_count, chain_byte_count)
 
-    def __clear_chain_info(self):
+    def __clear_chain_info(self) -> None:
         """Disassociate from all known chains
         """
         for chain in self.__chain_map.values():
@@ -491,7 +495,7 @@ class IptablesPacketFilterTable:
         """Initialize the attributes of this :class:`IptablesPacketFilterTable`
         from the output of **iptables(8)**
 
-        :param iptables_output: the output of ``iptables -L -xnv` as
+        :param iptables_output: the output of ``iptables -L -xnv`` as
             included in ``subprocess.CompletedProcess.stdout``
         :param add_builtins: if this parameter is ``True`` then for any
             builtin chain that is not in the ``iptables_output`` (or
@@ -640,14 +644,15 @@ class IptablesPacketFilterTable:
         self.__chain_map.pop(real_chain_name, None)
         chain._clear_pft()              # pylint: disable=protected-access
 
-    def zero_counters(self, chain: Optional[Chain] =None) -> None:
+    def zero_counters(self, chain: Optional[Chain] =None,
+                                rulenum: Optional[int] =None) -> None:
         """Zero the packet and byte counters for the specified chain
-        in the kernel. If no chain is specified, all chain counters are
-        zero'd.
+        or chain rule in the kernel. If no chain is specified, all chain
+        counters are zero'd.
 
         Note that this method does not affect the counters of :class:`Chain`
-        objects which will maintain the values they had from the last time
-        the system configuration was read.
+        or :class:`ChainRule` objects which will maintain the values they
+        had from the last time the system configuration was read.
         """
         cmd = ['-Z']
         if chain is not None:
@@ -656,6 +661,8 @@ class IptablesPacketFilterTable:
                     'attempt to zero chain counters of '
                     'chain owned by another table')
             cmd.append(chain.get_real_name())
+            if rulenum is not None:
+                cmd.append(str(rulenum))
         _ = self.iptables_run(cmd, check=True)
 
 # pylint: enable=too-many-instance-attributes, too-many-public-methods
