@@ -25,5 +25,7 @@ from .logtarget import LogTarget
 from .nattarget import SnatTarget, DnatTarget
 from .marktarget import MarkTarget
 from .masqueradetarget import MasqueradeTarget
+from .notracktarget import NoTrackTarget
 from .rejecttarget import RejectTarget
+from .tracetarget import TraceTarget
 from .ttltarget import TtlTarget
