@@ -44,7 +44,9 @@ from linuxnet.iptables import (
                 MarkTarget, ConnmarkTarget,
                 RejectTarget,
                 MasqueradeTarget,
+                NoTrackTarget,
                 TtlTarget,
+                TraceTarget,
                 # Matches
                 CommentMatch,
                 TcpmssMatch, TcpMatch,
@@ -74,9 +76,6 @@ class SimulatedIptablesRun:     # pylint: disable=too-few-public-methods
         # A run is a list of ExecutedCommand instances
         self.__run = []
 
-    def get_cmd(self):
-        return self.__cmd
-
     def get_run(self):
         return self.__run
 
@@ -216,23 +215,24 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 2)
-        match_list = rules[0].get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[0]
-        self.assertTrue(match.fragment().is_positive())
-        src = match.source_address()
-        self.assertTrue(src.is_positive() and
-                src.get_value() == IPv4Network('127.0.0.0/8'))
-        match_list = rules[1].get_match_list()
-        match = match_list[0]
-        prot = match.protocol()
-        self.assertTrue(prot.is_positive() and
-                prot.get_value() == 'udp')
-        dest = match.dest_address()
-        self.assertTrue(dest.is_positive() and
-                dest.get_value() == IPv4Network('10.10.0.0/16'))
+        self.assertEqual(input_chain.get_rule_count(), 2)
+        for rule in input_chain:
+            if rule.get_rulenum() == 1:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                self.assertTrue(match.fragment().is_positive())
+                src = match.source_address()
+                self.assertTrue(src.is_positive() and
+                    src.get_value() == IPv4Network('127.0.0.0/8'))
+            elif rule.get_rulenum() == 2:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                prot = match.protocol()
+                self.assertTrue(prot.is_positive() and
+                    prot.get_value() == 'udp')
+                dest = match.dest_address()
+                self.assertTrue(dest.is_positive() and
+                    dest.get_value() == IPv4Network('10.10.0.0/16'))
 
     def test_parsing_packet_type_match(self):
         """Parse output with packet type match
@@ -246,8 +246,8 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rules = input_chain.get_rules()
-        match_list = rules[0].get_match_list()
+        first_rule = next(iter(input_chain))
+        match_list = first_rule.get_match_list()
         self.assertEqual(len(match_list), 2)
         match = match_list[1]
         self.assertTrue(isinstance(match, PacketTypeMatch))
@@ -262,18 +262,17 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
 Chain INPUT (policy DROP 0 packets, 0 bytes)
     pkts      bytes target     prot opt in     out     source               destination
        8      524 DROP  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcpmss match 1:500
-    2182   251300 DROP  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp flags:!0x17/0x02
-      17      732 ACCEPT  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp dpt:22
+    2182   251300 DROP  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp flags:!0x17/0x02 option=10
+      17      732 ACCEPT  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp dpt:22 option=!10
 """ + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 3)
-        for rule in rules:
+        self.assertEqual(input_chain.get_rule_count(), 3)
+        for rule in input_chain:
             rulenum = rule.get_rulenum()
             match_list = rule.get_match_list()
-            # There is a PacketMatch matching prot
+            # There is a PacketMatch matching the packet protocol
             self.assertEqual(len(match_list), 2)
             match = match_list[-1]
             if rulenum == 1:
@@ -286,11 +285,17 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
                 flags = match.tcp_flags()
                 self.assertTrue(flags.is_syn_only() and
                                         not flags.is_positive())
+                tcp_option = match.tcp_option()
+                self.assertTrue(tcp_option.get_value() == 10 and
+                                tcp_option.is_positive())
             elif rulenum == 3:
                 self.assertTrue(isinstance(match, TcpMatch))
                 dport = match.dest_port()
                 self.assertTrue(dport.is_positive() and
                         dport.get_value()[0] == 22)
+                tcp_option = match.tcp_option()
+                self.assertTrue(tcp_option.get_value() == 10 and
+                                not tcp_option.is_positive())
 
     def test_parsing_udp_match(self):
         """Parse output with match related to UDP
@@ -298,15 +303,14 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         output = """\
 Chain INPUT (policy DROP 0 packets, 0 bytes)
     pkts      bytes target     prot opt in     out     source               destination
-      17      732 ACCEPT  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           udp dpt:53
+      17      732 ACCEPT  udp  --  *      *       0.0.0.0/0            0.0.0.0/0           udp dpt:53
 """ + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 1)
-        rule = rules[0]
-        match_list = rule.get_match_list()
+        self.assertEqual(input_chain.get_rule_count(), 1)
+        first_rule = next(iter(input_chain))
+        match_list = first_rule.get_match_list()
         # There is a PacketMatch matching prot
         self.assertEqual(len(match_list), 2)
         match = match_list[-1]
@@ -329,60 +333,48 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 4)
-        #
-        # Verify first rule
-        #
-        rule = rules[0]
-        match_list = rule.get_match_list()
-        # There is a PacketMatch matching prot
-        self.assertEqual(len(match_list), 2)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, IcmpMatch))
-        icmp_type = match.icmp_type()
-        self.assertTrue(icmp_type.is_positive() and
-                        icmp_type.get_type_name() == 'echo-request' and
-                        icmp_type.get_type_value() == 8)
-        #
-        # Verify second rule
-        #
-        rule = rules[1]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 2)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, IcmpMatch))
-        icmp_type = match.icmp_type()
-        self.assertTrue(not icmp_type.is_positive() and
-                        icmp_type.get_type_name() == 'any')
-        #
-        # Verify third rule
-        #
-        rule = rules[2]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 3)
-        match = match_list[-2]
-        self.assertTrue(isinstance(match, IcmpMatch))
-        icmp_type = match.icmp_type()
-        self.assertTrue(icmp_type.is_positive() and
-                        icmp_type.get_type_value() == 3)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, IcmpMatch))
-        icmp_type = match.icmp_type()
-        self.assertTrue(not icmp_type.is_positive() and
-                        icmp_type.get_type_value() == 3 and
-                        icmp_type.get_code() == 1)
-        #
-        # Verify fourth rule
-        #
-        rule = rules[3]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 2)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, IcmpMatch))
-        icmp_type = match.icmp_type()
-        self.assertTrue(icmp_type.is_positive() and
-                        icmp_type.get_type_value() == 3)
+        self.assertEqual(input_chain.get_rule_count(), 4)
+        for rule in input_chain:
+            if rule.get_rulenum() == 1:
+                match_list = rule.get_match_list()
+                # There is a PacketMatch matching prot
+                self.assertEqual(len(match_list), 2)
+                match = match_list[-1]
+                self.assertTrue(isinstance(match, IcmpMatch))
+                icmp_type = match.icmp_type()
+                self.assertTrue(icmp_type.is_positive() and
+                    icmp_type.get_type_name() == 'echo-request' and
+                    icmp_type.get_type_value() == 8)
+            elif rule.get_rulenum() == 2:
+                match_list = rule.get_match_list()
+                self.assertEqual(len(match_list), 2)
+                match = match_list[-1]
+                self.assertTrue(isinstance(match, IcmpMatch))
+                icmp_type = match.icmp_type()
+                self.assertTrue(not icmp_type.is_positive() and
+                                icmp_type.get_type_name() == 'any')
+            elif rule.get_rulenum() == 3:
+                match_list = rule.get_match_list()
+                self.assertEqual(len(match_list), 3)
+                match = match_list[-2]
+                self.assertTrue(isinstance(match, IcmpMatch))
+                icmp_type = match.icmp_type()
+                self.assertTrue(icmp_type.is_positive() and
+                                icmp_type.get_type_value() == 3)
+                match = match_list[-1]
+                self.assertTrue(isinstance(match, IcmpMatch))
+                icmp_type = match.icmp_type()
+                self.assertTrue(not icmp_type.is_positive() and
+                                icmp_type.get_type_value() == 3 and
+                                icmp_type.get_code() == 1)
+            elif rule.get_rulenum() == 4:
+                match_list = rule.get_match_list()
+                self.assertEqual(len(match_list), 2)
+                match = match_list[-1]
+                self.assertTrue(isinstance(match, IcmpMatch))
+                icmp_type = match.icmp_type()
+                self.assertTrue(icmp_type.is_positive() and
+                                icmp_type.get_type_value() == 3)
 
     def test_parsing_owner_match(self):
         """Parse output with match related to UID/GID
@@ -400,67 +392,50 @@ Chain OUTPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         output_chain = pft.get_builtin_chain('OUTPUT')
-        rules = output_chain.get_rules()
-        self.assertEqual(len(rules), 5)
-        #
-        # Verify first rule
-        #
-        rule = rules[0]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        uid = match.uid()
-        self.assertTrue(uid.is_positive() and uid.get_value() == (100, None))
-        #
-        # Verify second rule
-        #
-        rule = rules[1]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        gid = match.gid()
-        self.assertTrue(not gid.is_positive() and gid.get_value() == (100, None))
-        #
-        # Verify third rule
-        #
-        rule = rules[2]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 2)
-        match = match_list[-2]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        uid = match.uid()
-        self.assertTrue(uid.is_positive() and uid.get_value() == (100, 300))
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        uid = match.uid()
-        self.assertTrue(not uid.is_positive() and uid.get_value() == (200, None))
-        #
-        # Verify fourth rule
-        #
-        rule = rules[3]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        socket_exists = match.socket_exists()
-        self.assertTrue(socket_exists.is_positive())
-        #
-        # Verify fifth rule
-        # Note that a single match is expected because the owner
-        # match criteria are different.
-        #
-        rule = rules[4]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[-1]
-        self.assertTrue(isinstance(match, OwnerMatch))
-        gid = match.gid()
-        socket_exists = match.socket_exists()
-        self.assertTrue(
-                gid.is_positive() and gid.get_value() == (100, None) and
-                not socket_exists.is_positive())
+        self.assertEqual(output_chain.get_rule_count(), 5)
+        for rule in output_chain:
+            if rule.get_rulenum() == 1:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                self.assertTrue(isinstance(match, OwnerMatch))
+                uid = match.uid()
+                self.assertTrue(uid.is_positive() and uid.get_value() == (100, None))
+            elif rule.get_rulenum() == 2:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                self.assertTrue(isinstance(match, OwnerMatch))
+                gid = match.gid()
+                self.assertTrue(not gid.is_positive() and gid.get_value() == (100, None))
+            elif rule.get_rulenum() == 3:
+                match_list = rule.get_match_list()
+                self.assertEqual(len(match_list), 2)
+                match = match_list[-2]
+                self.assertTrue(isinstance(match, OwnerMatch))
+                uid = match.uid()
+                self.assertTrue(uid.is_positive() and uid.get_value() == (100, 300))
+                match = match_list[-1]
+                self.assertTrue(isinstance(match, OwnerMatch))
+                uid = match.uid()
+                self.assertTrue(not uid.is_positive() and uid.get_value() == (200, None))
+            elif rule.get_rulenum() == 4:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                self.assertTrue(isinstance(match, OwnerMatch))
+                socket_exists = match.socket_exists()
+                self.assertTrue(socket_exists.is_positive())
+            elif rule.get_rulenum() == 5:
+                #
+                # Note that a single match is expected because the owner
+                # match criteria are different.
+                #
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                self.assertTrue(isinstance(match, OwnerMatch))
+                gid = match.gid()
+                socket_exists = match.socket_exists()
+                self.assertTrue(
+                        gid.is_positive() and gid.get_value() == (100, None) and
+                        not socket_exists.is_positive())
 
     def test_parsing_connmark_match(self):
         """Parse output with connmark match
@@ -474,12 +449,10 @@ Chain PREROUTING (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('mangle')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('PREROUTING')
-        rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 1)
-        rule = rules[0]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[-1]
+        self.assertEqual(input_chain.get_rule_count(), 1)
+        rule = next(iter(input_chain))
+        self.assertEqual(rule.get_match_count(), 1)
+        match = next(iter(rule))
         self.assertTrue(isinstance(match, ConnmarkMatch))
         cmark = match.mark()
         self.assertTrue(cmark.is_positive() and cmark.get_value() == (0, None))
@@ -496,10 +469,9 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rule = input_chain.get_rules()[0]
-        match_list = rule.get_match_list()
-        self.assertEqual(len(match_list), 1)
-        match = match_list[0]
+        rule = next(iter(input_chain))
+        self.assertEqual(rule.get_match_count(), 1)
+        match = next(iter(rule))
         limit = match.limit()
         self.assertTrue(limit.is_positive() and
                 limit.get_value() == LimitMatch.Rate(15, LimitMatch.Rate.PER_MIN))
@@ -520,27 +492,77 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        for rule in input_chain.get_rules():
+        for rule in input_chain:
             rulenum = rule.get_rulenum()
             if rulenum == 1:
-                for match in rule.get_match_list():
+                for match in rule:
                     if isinstance(match, CommentMatch):
                         comment = match.comment().get_value()
                         self.assertEqual(comment, 'my comment')
                         break
             elif rulenum == 2:
-                for match in rule.get_match_list():
+                for match in rule:
                     if isinstance(match, CommentMatch):
                         comment = match.comment().get_value()
                         self.assertEqual(comment, 'another comment')
                         break
             elif rulenum == 3:
                 cit = iter(['another comment', 'foo bar'])
-                for match in rule.get_match_list():
+                for match in rule:
                     if isinstance(match, CommentMatch):
                         comment = match.comment().get_value()
                         self.assertEqual(comment, next(cit))
 
+    def test_parsing_addrtype_match(self):
+        """Parse output with ADDRTYPE match
+        """
+        output = """\
+Chain INPUT (policy DROP 0 packets, 0 bytes)
+    pkts      bytes target     prot opt in     out     source               destination
+       0        0 DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           ADDRTYPE match src-type UNICAST dst-type !UNICAST
+       0        0 LOG        all  --  *      *       0.0.0.0/0            0.0.0.0/0           ADDRTYPE match src-type !BLACKHOLE dst-type !UNICAST limit-in LOG flags 0 level 4 prefix `ADDR '
+       0        0 LOG        all  --  *      *       0.0.0.0/0            0.0.0.0/0           ADDRTYPE match src-type MULTICAST limit-out LOG flags 0 level 4 prefix `MCAST '
+       0        0 LOG        all  --  *      *       0.0.0.0/0            0.0.0.0/0           ADDRTYPE match src-type PROHIBIT /* PROHIBIT */ LOG flags 0 level 4 prefix `PROH '
+""" + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
+        pft = IptablesPacketFilterTable('filter')
+        self.assertTrue(pft.init_from_output(output))
+        input_chain = pft.get_builtin_chain('INPUT')
+        for rule in input_chain:
+            rulenum = rule.get_rulenum()
+            if rulenum == 1:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                src_addr_type = match.src_addr_type()
+                self.assertTrue(src_addr_type.is_positive() and
+                                src_addr_type.get_value() == 'UNICAST')
+                dst_addr_type = match.dst_addr_type()
+                self.assertTrue(not dst_addr_type.is_positive() and
+                                dst_addr_type.get_value() == 'UNICAST')
+            elif rulenum == 2:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                src_addr_type = match.src_addr_type()
+                self.assertTrue(not src_addr_type.is_positive() and
+                                src_addr_type.get_value() == 'BLACKHOLE')
+                dst_addr_type = match.dst_addr_type()
+                self.assertTrue(not dst_addr_type.is_positive() and
+                                dst_addr_type.get_value() == 'UNICAST')
+                self.assertTrue(match.limit_iface_in().is_positive())
+            elif rulenum == 3:
+                self.assertEqual(rule.get_match_count(), 1)
+                match = next(iter(rule))
+                src_addr_type = match.src_addr_type()
+                self.assertTrue(src_addr_type.is_positive() and
+                                src_addr_type.get_value() == 'MULTICAST')
+                self.assertTrue(match.limit_iface_out().is_positive())
+            elif rulenum == 4:
+                match_list = rule.get_match_list()
+                self.assertEqual(len(match_list), 2)
+                match = match_list[0]
+                src_addr_type = match.src_addr_type()
+                self.assertTrue(src_addr_type.is_positive() and
+                                src_addr_type.get_value() == 'PROHIBIT')
+
     def test_parsing_unknown_match(self):
         """Parse output with unknown match
         """
@@ -596,7 +618,7 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rule = input_chain.get_rules()[0]
+        rule = next(iter(input_chain))
         target = rule.get_target()
         self.assertEqual(target.get_log_prefix(), 'DROP-INPUT: ')
         self.assertEqual(target.get_log_level(), '6')
@@ -617,7 +639,7 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        rule = input_chain.get_rules()[0]
+        rule = next(iter(input_chain))
         target = rule.get_target()
         self.assertTrue(isinstance(target, RejectTarget))
         self.assertEqual(target.get_rejection_message(),
@@ -638,37 +660,39 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
-        # Verify SET
-        rule = input_chain.get_rules()[0]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MarkTarget))
-        self.assertEqual(target.get_op(), MarkTarget.SET)
-        self.assertEqual(target.get_mark(), 0xf)
-        # Verify XSET
-        rule = input_chain.get_rules()[1]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MarkTarget))
-        self.assertEqual(target.get_op(), MarkTarget.XSET)
-        self.assertEqual(target.get_mark(), 0x11)
-        self.assertEqual(target.get_mask(), 0xffff0011)
-        # Verify XOR
-        rule = input_chain.get_rules()[2]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MarkTarget))
-        self.assertEqual(target.get_op(), MarkTarget.XOR)
-        self.assertEqual(target.get_mask(), 0xf)
-        # Verify OR
-        rule = input_chain.get_rules()[3]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MarkTarget))
-        self.assertEqual(target.get_op(), MarkTarget.OR)
-        self.assertEqual(target.get_mask(), 0xf)
-        # Verify AND
-        rule = input_chain.get_rules()[4]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MarkTarget))
-        self.assertEqual(target.get_op(), MarkTarget.AND)
-        self.assertEqual(target.get_mask(), 0xff)
+        for rule in input_chain:
+            rulenum = rule.get_rulenum()
+            if rulenum == 1:
+                # Verify SET
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, MarkTarget))
+                self.assertEqual(target.get_op(), MarkTarget.SET)
+                self.assertEqual(target.get_mark(), 0xf)
+            elif rulenum == 2:
+                # Verify XSET
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, MarkTarget))
+                self.assertEqual(target.get_op(), MarkTarget.XSET)
+                self.assertEqual(target.get_mark(), 0x11)
+                self.assertEqual(target.get_mask(), 0xffff0011)
+            elif rulenum == 3:
+                # Verify XOR
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, MarkTarget))
+                self.assertEqual(target.get_op(), MarkTarget.XOR)
+                self.assertEqual(target.get_mask(), 0xf)
+            elif rulenum == 4:
+                # Verify OR
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, MarkTarget))
+                self.assertEqual(target.get_op(), MarkTarget.OR)
+                self.assertEqual(target.get_mask(), 0xf)
+            elif rulenum == 5:
+                # Verify AND
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, MarkTarget))
+                self.assertEqual(target.get_op(), MarkTarget.AND)
+                self.assertEqual(target.get_mask(), 0xff)
 
     def test_parsing_connmark_target(self):
         """Parse output with CONNMARK target
@@ -684,22 +708,25 @@ Chain PREROUTING (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('mangle')
         self.assertTrue(pft.init_from_output(output))
         prerouting_chain = pft.get_builtin_chain('PREROUTING')
-        rule = prerouting_chain.get_rules()[0]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, ConnmarkTarget))
-        self.assertEqual(target.get_mark(), 0x11)
-        rule = prerouting_chain.get_rules()[1]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, ConnmarkTarget) and
+        self.assertEqual(prerouting_chain.get_rule_count(), 3)
+        for rule in prerouting_chain:
+            rulenum = rule.get_rulenum()
+            if rulenum == 1:
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, ConnmarkTarget))
+                self.assertEqual(target.get_mark(), 0x11)
+            elif rulenum == 2:
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, ConnmarkTarget) and
                                 target.is_saving_mark())
-        self.assertEqual(target.get_nfmask(), 0xfffff)
-        self.assertEqual(target.get_ctmask(), 0x1f)
-        rule = prerouting_chain.get_rules()[2]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, ConnmarkTarget) and
+                self.assertEqual(target.get_nfmask(), 0xfffff)
+                self.assertEqual(target.get_ctmask(), 0x1f)
+            elif rulenum == 3:
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, ConnmarkTarget) and
                                 target.is_restoring_mark())
-        self.assertEqual(target.get_nfmask(), 0xfffff)
-        self.assertEqual(target.get_ctmask(), 0x1f)
+                self.assertEqual(target.get_nfmask(), 0xfffff)
+                self.assertEqual(target.get_ctmask(), 0x1f)
 
     def test_parsing_ttl_target(self):
         """Parse output with TTL target
@@ -715,17 +742,18 @@ Chain PREROUTING (policy DROP 0 packets, 0 bytes)
         pft = IptablesPacketFilterTable('mangle')
         self.assertTrue(pft.init_from_output(output))
         prerouting_chain = pft.get_builtin_chain('PREROUTING')
-        rule = prerouting_chain.get_rules()[0]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, TtlTarget) and
+        for rule in prerouting_chain:
+            rulenum = rule.get_rulenum()
+            target = rule.get_target()
+            if rulenum == 1:
+                self.assertTrue(isinstance(target, TtlTarget) and
                                 target.get_ttl_value() == 10)
-        rule = prerouting_chain.get_rules()[1]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, TtlTarget) and
+            elif rulenum == 2:
+                self.assertTrue(isinstance(target, TtlTarget) and
                                 target.get_ttl_inc() == 1)
-        rule = prerouting_chain.get_rules()[2]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, TtlTarget) and
+            elif rulenum == 3:
+                target = rule.get_target()
+                self.assertTrue(isinstance(target, TtlTarget) and
                                 target.get_ttl_dec() == 2)
 
     def test_parsing_snat_target(self):
@@ -739,7 +767,7 @@ Chain POSTROUTING (1 references)
         pft = IptablesPacketFilterTable('nat')
         self.assertTrue(pft.init_from_output(output))
         postrouting_chain = pft.get_builtin_chain('POSTROUTING')
-        rule = postrouting_chain.get_rules()[0]
+        rule = next(iter(postrouting_chain))
         target = rule.get_target()
         self.assertEqual(target.get_target_name(), 'SNAT')
         self.assertEqual(target.get_address(), IPv4Address('10.10.10.18'))
@@ -759,27 +787,56 @@ Chain POSTROUTING (1 references)
         pft = IptablesPacketFilterTable('nat')
         self.assertTrue(pft.init_from_output(output))
         postrouting_chain = pft.get_builtin_chain('POSTROUTING')
-        rule = postrouting_chain.get_rules()[0]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MasqueradeTarget))
-        rule = postrouting_chain.get_rules()[1]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MasqueradeTarget) and
+        self.assertEqual(postrouting_chain.get_rule_count(), 5)
+        for rule in postrouting_chain:
+            rulenum = rule.get_rulenum()
+            target = rule.get_target()
+            if rulenum == 1:
+                self.assertTrue(isinstance(target, MasqueradeTarget))
+            elif rulenum == 2:
+                self.assertTrue(isinstance(target, MasqueradeTarget) and
                                 target.uses_random_port_mapping())
-        rule = postrouting_chain.get_rules()[2]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MasqueradeTarget) and
+            elif rulenum == 3:
+                self.assertTrue(isinstance(target, MasqueradeTarget) and
                                 target.get_ports() == (2000, None))
-        rule = postrouting_chain.get_rules()[3]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MasqueradeTarget) and
+            elif rulenum == 4:
+                self.assertTrue(isinstance(target, MasqueradeTarget) and
                             target.get_ports() == (2000, None) and
                                 target.uses_random_port_mapping())
-        rule = postrouting_chain.get_rules()[4]
-        target = rule.get_target()
-        self.assertTrue(isinstance(target, MasqueradeTarget) and
+            elif rulenum == 5:
+                self.assertTrue(isinstance(target, MasqueradeTarget) and
                                 target.get_ports() == (20000, 30000))
 
+    def test_parsing_notrack_target(self):
+        """Parse output with NOTRACK target
+        """
+        output = (self.EMPTY_PREROUTING + '\n' + """\
+Chain OUTPUT (policy ACCEPT 12 packets, 1845 bytes)
+ pkts bytes target     prot opt in     out     source               destination
+    0     0 NOTRACK    tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp dpt:55555
+""")
+        pft = IptablesPacketFilterTable('raw')
+        self.assertTrue(pft.init_from_output(output))
+        output_chain = pft.get_builtin_chain('OUTPUT')
+        rule = next(iter(output_chain))
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, NoTrackTarget))
+
+    def test_parsing_trace_target(self):
+        """Parse output with TRACE target
+        """
+        output = (self.EMPTY_PREROUTING + '\n' + """\
+Chain OUTPUT (policy ACCEPT 12 packets, 1845 bytes)
+ pkts bytes target     prot opt in     out     source               destination
+    0     0 TRACE      tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           tcp dpt:55555
+""")
+        pft = IptablesPacketFilterTable('raw')
+        self.assertTrue(pft.init_from_output(output))
+        output_chain = pft.get_builtin_chain('OUTPUT')
+        rule = next(iter(output_chain))
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, TraceTarget))
+
     def test_parsing_unknown_target(self):
         """Parse output with unknown target
         """
@@ -936,10 +993,20 @@ Chain prod_OUTPUT (1 references)
         pft = IptablesPacketFilterTable('filter', runner=runner)
         init_ok = pft.init_from_output(self.GOOD_OUTPUT)
         self.assertTrue(init_ok)
-        chain = pft.get_builtin_chain('OUTPUT')
         #
         # Zero the counters of a specific chain
         #
+        chain = pft.get_builtin_chain('FORWARD')
+        rule = next(iter(chain))
+        rule.zero_counters()
+        expected_cmd = ['iptables', '-t', 'filter', '-Z', 'FORWARD', '1']
+        commands = runner.get_run()
+        self.assertEqual(commands[0].cmd, expected_cmd)
+        #
+        # Zero the counters of a specific chain
+        #
+        runner.clear_run()
+        chain = pft.get_builtin_chain('OUTPUT')
         chain.zero_counters()
         expected_cmd = ['iptables', '-t', 'filter', '-Z', 'OUTPUT']
         commands = runner.get_run()
@@ -1013,7 +1080,7 @@ Chain OUTPUT (policy ACCEPT 30 packets, 3000 bytes)
 
 
 class TestChainOperations(unittest.TestCase):
-    """Test chain operations. The runner is a no-op
+    """Test chain operations.
     """
 
     @staticmethod
@@ -1111,6 +1178,80 @@ class TestChainOperations(unittest.TestCase):
         self.assertEqual(len(chain2.get_rules()), 0)
         self.assertEqual(len(chain3.get_rules()), 0)
 
+    def test_rule_enumeration(self):
+        """Rule enumeration
+        """
+        runner = SimulatedIptablesRun(0, None)
+        pft = IptablesPacketFilterTable('filter', runner=runner)
+        #
+        # We create the following structure:
+        #
+        #  jump_target:
+        #  goto_target:
+        #  caller_chain:
+        #       -m owner --uid-owner 10 --gid-owner 1 -j RETURN (1 match)
+        #       -p tcp --dport 22 -j jump_target                (2 matches, chain-target)
+        #       -p tcp -j DROP                                  (1 match)
+        #       -p udp -g goto_target                           (1 match, chain-target-goto)
+        #       -j jump_target                                  (0 matches, chain-target)
+        #
+        jump_target = pft.create_chain('jump_target')
+        goto_target = pft.create_chain('goto_target')
+        chain = pft.create_chain('caller_chain')
+        chain.append_rule(ChainRule(
+                match=OwnerMatch().uid().equals(10).gid().equals(1),
+                target=Targets.RETURN))
+        chain.append_rule(ChainRule(
+                match_list=[
+                        PacketMatch().protocol().equals('tcp'),
+                        TcpMatch().dest_port().equals(22),
+                        ],
+                target=ChainTarget(chain=jump_target)))
+        chain.append_rule(ChainRule(
+                match=PacketMatch().protocol().equals('tcp'),
+                target=Targets.DROP))
+        chain.append_rule(ChainRule(
+                match=PacketMatch().protocol().equals('udp')
+                        ).go_to(chain=goto_target))
+        chain.append_rule(ChainRule(
+                target=ChainTarget(chain=jump_target)))
+        #
+        # 3 rules target chains
+        #
+        rules_to_chains = list(chain.iter_rules(chain_target=True))
+        self.assertEqual(len(rules_to_chains), 3)
+        #
+        # 1 rule uses goto
+        #
+        rules_go_to_chains = list(chain.iter_rules(uses_goto=True))
+        self.assertEqual(len(rules_go_to_chains), 1)
+        #
+        # 3 rules with a single match
+        #
+        single_match_rules = list(chain.iter_rules(match_count=1))
+        self.assertEqual(len(single_match_rules), 3)
+        #
+        # 1 rule with a single match and a chain target
+        #
+        single_match_rules_to_chains = list(
+                chain.iter_rules(chain_target=True, match_count=1))
+        self.assertEqual(len(single_match_rules_to_chains), 1)
+        #
+        # 1 rule with a match for TCP dest port 22
+        #
+        port22_rules = list(chain.iter_rules(match=TcpMatch().dest_port().equals(22)))
+        self.assertEqual(len(port22_rules), 1)
+        #
+        # 1 rule with a PacketMatch that uses goto
+        #
+        packet_goto_rules = list(chain.iter_rules(match=PacketMatch(), uses_goto=True))
+        self.assertEqual(len(packet_goto_rules), 1)
+        #
+        # 1 rule with a match with 2 criteria
+        #
+        rules = list(chain.iter_rules(match=OwnerMatch().uid().any().gid().equals(1)))
+        self.assertEqual(len(rules), 1)
+
     def test_rule_operations(self):
         """Rule search, deletion
         """
